

# 1. os 모듈의 파일과 디렉터리 관련 함수
import os

# 기본 디렉터리 지정 
os.chdir('C:/PythonStatistics') 

# 현재 작업 디렉터리 경로
print(os.getcwd()) # 'C:\PythonStatistics'

# 작업 디렉터리 변경
os.chdir('./chap05_File_Numpy')
print(os.getcwd()) # 'C:\PythonStatistics\chap05_Numpy'

# 현재 작업 디렉터리 목록
print(os.listdir('.')) # list 반환 - ['exams', 'step01_file_system.py', 'step02_csv_excel_FileIO.py', 'step03_array_zeros_arange.py', 'step04_indexing.py', 'step05_axis.py', 'step05_random.py', 'step06_linear_algebra1.py', 'step06_linear_algebra2.py', 'test2']

# 디렉터리 생성
os.mkdir('test')
print(os.listdir('.'))

# 디렉터리 이동
os.chdir('test')
print(os.getcwd()) # 'C:\PythonStatistics\chap05_Numpy\test'

# 상위 디렉터리 이동
os.chdir('../')

# 디렉터리 삭제
os.rmdir('test')
print(os.listdir('.'))

# 여러 디렉터리 생성
os.makedirs('test2/test3')

# 디렉터리 이동
os.chdir('test2')
print(os.listdir('.')) # ['test3']


# 상위 디렉터리 이동
os.chdir('../')
print(os.getcwd()) # C:\PythonStatistics\chap05_Numpy

# 여러 개의 디렉터리 삭제
os.removedirs('test2/test3')
print(os.listdir('.')) 



# 2. os.path 모듈의 파일 경로 관련 함수
import os.path # windows 파일 경로를 조작하는 모듈

print(dir(os.path))
'''
['abspath', 'altsep', 'basename', 'commonpath', 'commonprefix', 'curdir', 'defpath', 'devnull', 'dirname', 'exists', 'expanduser', 'expandvars', 'extsep', 'genericpath', 'getatime', 'getctime', 'getmtime', 'getsize', 'isabs', 'isdir', 'isfile', 'islink', 'ismount', 'join', 'lexists', 'normcase', 'normpath', 'os', 'pardir', 'pathsep', 'realpath', 'relpath', 'samefile', 'sameopenfile', 'samestat', 'sep', 'split', 'splitdrive', 'splitext', 'stat', 'supports_unicode_filenames', 'sys']
'''

# 기본 디렉터리 지정  
os.chdir('C:/PythonStatistics/chap05_File_Numpy')
print(os.getcwd()) 

# 파일 절대경로 
print(os.path.abspath('step01_file_system.py'))

# 디렉터리명 반환 
print(os.path.dirname('chap05_File_Numpy/step01_file_system.py'))#''chap05_Numpy'

# 파일 경로 존재 여부 확인  
print(os.path.exists('C:/PythonStatistics/chap05_Numpy')) # True

# 파일 존재 여부 확인 
print(os.path.isfile('step01_file_system.py')) # True
print(os.path.isdir('test')) # False

print(os.path.split("c:\\test\\test1.txt")) # ('c:\\test', 'test1.txt')
print(os.path.join('c:\\test', 'test1.txt')) # 'c:\\test\\test1.txt'

print(os.path.getsize('step01_file_system.py')) # 1883 byte



# 3. glob 모듈의 파일 경로 관련 함수
import glob
help(glob)
#  glob - Filename globbing utility.
'''
NAME
    glob - Filename globbing utility.
FUNCTIONS
    escape(pathname)
        Escape all special characters.
    
    glob(pathname, *, recursive=False)
        Return a list of paths matching a pathname pattern.
        
        The pattern may contain simple shell-style wildcards a la
        fnmatch. However, unlike fnmatch, filenames starting with a
        dot are special cases that are not matched by '*' and '?'
        patterns.
        
        If recursive is true, the pattern '**' will match any files and
        zero or more directories and subdirectories.
    
    iglob(pathname, *, recursive=False)
        Return an iterator which yields the paths matching a pathname pattern.
        
        The pattern may contain simple shell-style wildcards a la
        fnmatch. However, unlike fnmatch, filenames starting with a
        dot are special cases that are not matched by '*' and '?'
        patterns.
        
        If recursive is true, the pattern '**' will match any files and
        zero or more directories and subdirectories.
DATA
    __all__ = ['glob', 'iglob', 'escape']
FILE
    c:\python37\lib\glob.py
'''
print(dir(glob))
'''
'escape', 'fnmatch', 'glob', 'glob0', 'glob1', 'has_magic', 'iglob', 'magic_check', 'magic_check_bytes', 'os', 're']
'''

glob.escape()

'''
유니스 셀이 사용하는 규칙에 따라 지정된 패턴과 일치하는 모든 경로명을 찾는다.
결과는 무순서로 반환된다.
*, ?, []로 표시되는 문자 범위는 올바르게 일치한다.
리터럴 일치를 위해서는 대괄호 안에 메타문자를 넣는다. 예를 들면 '[?]'는 '?'
문자와 일치한다.

glob.glob(pathname, *, recursive=False) :
경로 지정을 포함하는 문자열인 pathname에 일치하는 경로 이름의 비어있을 수 있는 리스트를 반환한다.
pathname은 절대경로와 상대경로를 지정할 수 있다.

glob.iglob(pathname, *, recursive=False)
glob()과 같은 값을 산출하지만 이터레이터를 반환한다.

glob.escape(pathname)
모든 특수문자(*, ?, [])를 이스케이프 처리한다. 특수문자가 들어있을 수 있는 임의의 
리터럴 문자열을 일치시키려는 경우에 유용하다. 
 '''

glob.glob('./[0-9].*')
#['./10.test', './20.gif']
glob.glob('*.exe')
#[test.exe, 10.exe, a.exe, 1.exe]
glob.glob('?.exe')
#[a.exe, 1.exe]
glob.glob('*.txt', recursive=True)
#[2.txt, 1.txt]

