
import numpy as np 

# 1. 기본 행렬연산 관련함수

# 1) 단위행렬 : 대각원소가 1이고, 나머지는 모두 0인 n차 정방행렬
eye_mat = np.eye(3) 
print(eye_mat)
'''
[[1. 0. 0.]
 [0. 1. 0.]
 [0. 0. 1.]]
'''

# 2) 대각행렬 : 대각성분 이외의 모든 성분이 모두 '0'인 n차 정방행렬
x = np.arange(1,10).reshape(3,3)
print(x)
'''
[[1 2 3]
 [4 5 6]
 [7 8 9]]
'''

diag_vec = np.diag(x)
print(diag_vec)
# [1 5 9]
diag_mat = eye_mat * diag_vec
print(diag_mat)
'''
[[1. 0. 0.]
 [0. 5. 0.]
 [0. 0. 9.]]
'''

# 3) 대각합 : 정방행렬의 대각에 위치한 원소들의 합 
trace_scala = np.trace(diag_mat) # 2차원 
print(trace_scala) # 15.0


x = np.arange(1,13).reshape(2,2,3) # 3차원 
print(x)
'''
[[[ 1  2  3]
  [ 4  5  6]]

 [[ 7  8  9]
  [10 11 12]]]
'''

trace_vec = np.trace(x)
print(trace_vec) # [11 13 15]
'''
11=1+10
13=2+11
15=3+12
'''

# 4) 행렬식(행렬식의 판별식) : 역행렬 존재여부를 확인하는 방법으로 행렬식이 0이 아니면 역행렬 존재 
'''
역행렬 판별식으로 사용 : D(A) = ab - bc = 0이면 존재하지 않음 
'''
x = np.array([[3,4], [1,2]])
print(x)
'''
[[3 4]
 [1 2]]
'''

det_scala = np.linalg.det(x)
print(det_scala) # 2.0000000000000004
'''
A = [[a, b],
     [c, d]]

det = ad - bc
2 = (3x2) - (4x1)
'''

# 5) 역행렬
inv_mat = np.linalg.inv(x)
print(inv_mat)
'''
A = [[a, b],
     [c, d]] 일때
D = ad - bc라 하면
1) ab-bc !=0 이면 A^-1 존재
   즉 A^-1 = 1 / (ad - bc)[[d, -b],
                           [-c, a]]
0.5 [[2 -4]
     [-1 3]]
[[ 1.  -2. ]
 [-0.5  1.5]]

2) ab - bc = 0 이면 A^-1 존재하지 않음 
'''

# 원래 행렬 x와 역행렬을 행렬곱 연산 -> 단위행렬   
print(np.dot(x, inv_mat)) # 단위행렬 
'''
[[1.00000000e+00 0.00000000e+00]
 [1.11022302e-16 1.00000000e+00]]
'''


