
#================================================
# 1. 이산확률분포 : ppt.3~8
#================================================

'''
-확률변수 X가 가질 수 있는 값이 셀 수 있는 집합
-주사위를 던졌을 때 나오는 수, 동전의 앞면/뒷면(이항분포)

이항분포와 검정 : 2가지 범주를 갖는 이산확률분포 일종  
 - 이산확률분포 : 베르누이분포, 이항분포, 포아송분포
  ex) 베르누이분포, 이항분포 : 성공(1)/실패(0)
  ex) 포아송분포 : 특정한 사건이 발생할 가능성이 매우 적은 확률분포(번개에 맞을 확률)
 - 이항검정(binominal test) : 이항분포(2가지 범주)를 이용한 가설검정 
'''

### 1. 베르누이(Bernoulli) 분포
'''
 베르누이분포 : 확률변수 X가 가질 수 있는 값이 0과 1만 갖는 확률분포
 (모수 : p=성공확률) 
 베르누이 시행 : 이러한 분포를 따르는 확률변수의 시행(실험)
 확률변수 X의 집합 = {1, 0} : 값이 1이면 성공, 0이면 실패
 확률(p) : 1(성공)이 나오는 확률, 1 - p : 0(실패)가 나오는 확률
 
 확률 성질 
   1) 전체 합 = 1
   2) 0 <= p <= 1
''' 

from scipy import stats # 확률분포 검정 
import numpy as np
# 차트에서 한글 지원 
import matplotlib.pyplot as plt
plt.rcParams['font.family'] = 'Malgun Gothic'

# 음수 부호 지원 
import matplotlib
matplotlib.rcParams['axes.unicode_minus'] = False


# 1. 베르누이분포 객체 : 이항분포를 따르는 확률변수(이항변수)에서 성공이 나올 확률분포(모수 : 성공확률) 
#np.random.seed(0)

# 1) 확률변수 X : 이항분포  
X = np.array([0, 1]) # 확률변수 X 집합(표본값) = {0, 1}
print('X =',X)

# 2) 베르누이분포 객체 생성
p = 0.4 # 성공확률(모수)    
obj = stats.bernoulli(p) # 베르누이분포 객체 생성(모수=p) 
print(obj)
help(stats.bernoulli)

# 2. 확률질량함수 : 확률변수 X에 정의된 값을 확률에 대응시키는 함수
X_pmf = obj.pmf(X) 
#X_pmf = stats.bernoulli.pmf(X, p) # array([0.6, 0.4])-[실패, 성공]
print('X_pmf =', X_pmf)

print('성공 확률 =', obj.pmf(1)) # 0.4 - 성공(1) 확률 
print('실패 확률 =', obj.pmf(0)) # 0.6 - 실패(0) 확률


# 확률질량함수 시각화 
plt.bar(X, X_pmf)
plt.xlim(-1, 2)
plt.ylim(0, 1)
plt.xticks([0, 1], ["x=0", "x=1"]) # x축 눈금
plt.xlabel("확률변수의 집합(값)")
plt.ylabel("P(x)")
plt.title("베르누이 분포의 확률질량함수")
plt.show()

# 3. python 함수 정의 : 확률질량함수 정의
def Bern(p) :   
    def pmf(x) : # x= 0 or 1
        return p**x * (1-p)**(1-x) # 확률질량함수 수학식 
    return pmf

pmf = Bern(0.4) # Bern(성공확률 p)

x = np.array([0, 1]) # 확률변수 X의 집합(표본값) = {0,1} 
pmf = pmf(x) # 베르누이분포의 확률 반환  
print('pmf =', pmf) # array([0.6, 0.4])

plt.bar(x, pmf) # # 확률변수 X와 확률질량함수 
plt.xlim(-1, 2)
plt.ylim(0, 1)
plt.xticks([0, 1], ["x=0", "x=1"]) # index, x축 눈금의 label
plt.xlabel("확률 변수의 집합(값)")
plt.ylabel("P(x)")
plt.title("베르누이 분포의 확률질량함수")
plt.show()


# 4. 누적분포함수
X_cdf = obj.cdf(X) # array([0.6, 1. ])
X_cdf = stats.bernoulli.cdf(X, p, loc=0) # array([0.6, 1. ])
print('X_cdf =', X_cdf)

x = [-0.5, 0, 0.5, 1.5]
cdf = X_cdf.tolist()*2
cdf = sorted(cdf) #  오름차순 정렬
print('cdf =', cdf)

# 누적분포함수 시각화 
plt.plot(x, cdf, drawstyle='steps-post')
plt.xlim(-0.5, 1.5)
plt.ylim(0, 1.5)
plt.xticks([0, 1], ["x=0", "x=1"]) # index, x축 눈금의 label
plt.title("누적분포함수")
plt.xlabel("X")
plt.ylabel("F(X)")
plt.show()


# 5. 모집단 기댓값과 분산 : 주요 성질 
# 1) 모집단 기댓값 : E(X) = p
Ex = p # obj.mean() 
print('기댓값 =',Ex) # 기댓값 = 0.4

# 2) 모집단 분산 : VAR(X) = p(1-p)
VARx = p *(1-p) # obj.var()
print('분산 = ', VARx) # 분산 =  0.24

# 3) 확률변수 X 기술통계
print('확률변수 X 기술통계')
print(stats.describe(X))

'''
시행횟수, X 범위, 평균(성공=p 기댓값), 분산(p*(1-p)), 왜도, 첨도) 
DescribeResult(nobs=2, minmax=(0, 1), mean=0.5, variance=0.5, skewness=0.0, kurtosis=-2.0)
'''
#[해설] sample 수가 많을 수록 모집단의 모수(평균과 분산)에 근사해진다. 

