### 2. 이항(Binomial) 분포

from scipy import stats # 확률분포 검정 
import numpy as np # 수치 자료

# 차트에서 한글 지원 
import matplotlib.pyplot as plt
plt.rcParams['font.family'] = 'Malgun Gothic'

# 음수 부호 지원 
import matplotlib
matplotlib.rcParams['axes.unicode_minus'] = False

# 1. 이항분포 객체 생성 
# - 항분포를 따르는 확률변수(이항변수)에서 성공이 나올 확률분포(모수 : 성공확률) 

# 1) 이항분포 객체 생성
n, p = 10, 0.5 # 시행횟수(베르누이 시행 n), 확률(모수=기댓값)
obj = stats.binom(n, p) # 이항분포 객체 생성(모수=n,p)
print(obj)

# 2) 확률변수 X : 이항분포  
X = np.arange(n + 1) # 확률변수 X의 집합(표본값) = {0,1,...n} 
print('X =',X)

# 2. 확률질량함수 : f(x) -> p
# (1) 확률변수 X + 확률질량함수
X_pmf = obj.pmf(X) # 이항분포 확률 반환  
print('X_pmf =', X_pmf) # [0.00097656,...,0.24609375,...0.00097656]

print('모두 실패일 확률 =', obj.pmf(0)) # 모두 0인 확률 : 0.0009765625
print('5회 성공일 확률 =', obj.pmf(5)) # 5회 성공인 확률 : 0.24609375000000025
print('모두 성공일 확률 =', obj.pmf(10)) # 모두 1인 확률 : 0.0009765625
# [해설] 10번 시행중에서 모두 실패나 성공인 경우의 확률은 매우적다.
# 나올 수 있는 확률이 0.5이기 때문에 10번 시행 중에서 5번 성공 횟수가 가장 높다. 

# (2) 확률분포 
# 성공횟수에 따른 이항분포의 확률분포 
plt.bar(X, X_pmf) # 확률변수 x와 확률함수 이용 
plt.xlabel("확률변수의 집합")
plt.ylabel("P(x)")
plt.title("이항분포의 확률질량함수")
plt.show()
# [해설] 이항분포도 평균값을 기준으로 좌우대칭 분포를 보인다.

# 3. python 함수 정의 : 이항분포 확률질량함수 정의 
#from math import factorial 
from scipy.special import comb # 조합 nCx 연산 

# python 이항분포 확률함수(n=시행,p=확률 ->확률 반환)
def Bin(n, p):
    fx = []
    for x in range(n+1) : # 성공횟수(0 ~ 15) : 15번 시행
        nCx = comb(n, x) # 조합 nCx
        #ncx = factorial(n) / (factorial(x) * factorial(n - x))
        fx.append( nCx * p**x * (1-p)**(n-x) ) # 이항분포 확률함수 
    return fx

# (a) 확률 : p=0.5, 성공횟수 : n=10
fx = np.array(Bin(n, p)) # 확률함수 호출 
#plt.ylim(0, 0.3) # 차트 결과에 따라서 지정 
plt.text(8, 0.20, 'n, p = 10, 0.5') # 차트 결과에 따라서 지정
plt.xlabel("확률변수의 집합")
plt.ylabel("P(x)")
plt.title("이항분포의 확률질량함수")

x = np.arange(n+1) # 확률변수 x : 0~n번 성공횟수 
plt.bar(x, fx, color='darkblue')
plt.show()


# (b) 확률 p에 따른 확률분포 
fig = plt.figure()
chart = fig.add_subplot()
n = 100 # 성공횟수 
x = np.arange(n+1) # 확률변수 X
p = [0.3, 0.5, 0.7] # p=0.3, p=0.5, p=0.7
c = ['blue','red','green']
for p,c in zip(p, c) :
    fx = np.array(Bin(n, p)) 
    chart.bar(x, fx, color=c)
plt.title("확률(p)에 따른 확률분포")
plt.show()


# 4. 누적분포함수
n, p = 10, 0.5 # 시행횟수(베르누이 시행 n), 확률(모수=기댓값)
obj = stats.binom(n, p) # 이항분포 객체 생성(모수=n,p)

X = np.arange(n + 1) # 확률변수 X의 집합(표본값) = {0,1,...n} 

X_cdf = obj.cdf(X) # 1) binom 객체 이용
stats.binom.cdf(n, p) # 2) binom 클래스 이용
help(stats.binom)
help(stats.binom.cdf)

print('X_cdf =', X_cdf)

# 누적확률분포 
plt.plot(X, X_cdf)
plt.text(1, 0.8, 'n, p = 10, 0.5') # 차트 내 텍스트 표현 
plt.xlabel("확률변수의 집합")
plt.ylabel("P(x)")
plt.title("이항분포의 누적분포함수")
plt.show()


# 5. 모집단의 기댓값과 분산
n, p = 10, 0.5
X = np.arange(n + 1)

# 1) 모집단 기댓값 : E(X) = np
Ex = n*p 
print('기댓값 =',Ex) 

# 2) 모집단 분산 : VAR(X) = np(1-p)
VARx = n*p *(1-p)
print('분산 = ', VARx) 

# 3) 확률변수 X의 기술통계 
print('확률변수 X 기술통계량')
print(stats.describe(X))
'''
DescribeResult(nobs=1000, minmax=(0, 4), mean=0.881, variance=0.7015405405405405, skewness=0.7684897676063843, kurtosis=0.21244385790777187)
'''
#[해설] 시물레이션 수가 클 수록 모집단의 모수와 가까워진다. 


# 6. 표본 추출(random sampling) 
# size : N번 시뮬레이션(모집단(n,p) -> size개 random 샘플링)
obj.rvs(size=1) 
stats.binom.rvs(n, p, loc=0, size=1, random_state=None)#array([3])
# size=10
stats.binom.rvs(n, p, loc=0, size=10, random_state=None)
# array([6, 4, 3, 6, 5, 5, 6, 3, 4, 6])
# [해설] 시행횟수 n=10, 확률 p=0.5 일때 10번 시뮬레이션으로 임의추출 


# (a) 동전 : 이항분포의 모집단으로 부터 N개 표본 추출(sampling):N번 시뮬레이션
# n=독립시행회수(베르누이시행 n번), p=성공확률, size=N개 샘플링 
sample1 = stats.binom.rvs(n=1, p=0.5, size=10) # 동전 1번 시행, 나올 확률(0.5)
print('sample1 =', sample1)
# array([1, 1, 1, 0, 0, 0, 1, 0, 0, 0])

sample2 = stats.binom.rvs(n=5, p=0.5, size=10) # 동전 5번 시행, 나올 확률(0.5)  array([3, 3, 2, 5, 2, 3, 1, 3, 3, 2]) -> sampling에 따른 성공
print('sample2 =', sample2)
# 1회 sampling 시 성공(앞면)-3/5(0.6)
# 3회 sampling 시 성공(앞면)-2/5(0.4)
# 5회 sampling 시 성공(앞면)-5/5(1.0) 

# (b)주사위를 던져서 특정 눈금이 나오는 확률(ex:주사위 6번 던져서 2가 나올확률)  
stats.binom.rvs(n=6, p=1/6) #주사위 6번 시행, 확률(1/6)=성공확률 모두 동일
stats.binom.rvs(n=6, p=1/6, size=10) # 표본 10개 추출 
# array([2, 1, 1, 1, 2, 1, 0, 1, 0, 2])
# 1회 sampling 시 성공(앞면)-2/6(0.333) : 6번 던져서 2번 성공 
# 9회 sampling 시 성공(앞면)-0/6(0.0) : 6번 던져서 모두 실패 


# 5) 표본의 확률변수 X 통계량
# 1) 확률변수 x 만들기 : 베르누이 시행 N의 베르누이분포변수 X 생성
n, p = 5, 1/6 # 5번 시행 
X = stats.binom.rvs(n, p, size=1000) # 1000번 시뮬레이션 : 표본 추출  
X  # 성공횟수 

# 2) 모집단 기댓값 : E(X) = np
Ex = n*p 
print('기댓값 =',Ex) # 기댓값 = 0.8333333333333333

# 3) 모집단 분산 : VAR(X) = np(1-p)
VARx = n*p *(1-p)
print('분산 = ', VARx) # 분산 =  0.6944444444444444

# 4) 확률변수 X의 기술통계 
print('표본의 확률변수 X의 검정통계량')
print(stats.describe(X))
'''
DescribeResult(nobs=1000, minmax=(0, 4), mean=0.881, variance=0.7015405405405405, skewness=0.7684897676063843, kurtosis=0.21244385790777187)
'''
#[해설] 시물레이션 수가 클 수록 모집단의 모수와 가까워진다. 
