from scipy import stats # 확률분포와 검정
import numpy as np

# 1. 표본 추출(random sampling) 

# 1) 동전 확률실험 : n번 독립시행으로 앞면이 나올 확률

# 모집단의 모수 : n=독립시행회수(베르누이시행 n번), p=성공확률 
# size = N번 시뮬레이션 : (모집단의 모수(n,p) -> size개 random 샘플링)

sample1 = stats.binom.rvs(n=1, p=0.5, size=10) # 독립시행 1회, 성공확률(0.5)
print('sample1 =', sample1)
# array([1, 1, 1, 0, 0, 0, 1, 0, 0, 0])
# [해설] 독립시행 n=1회, 성공확률 p=0.5 일때 10번 시뮬레이션으로 표본 추출 
sample2 = stats.binom.rvs(n=5, p=0.5, size=10) # 독립시행 5회, 성공확률(0.5)
print('sample2 =', sample2)
# sample2 = [1 2 3 1 2 2 2 2 2 2] -> sampling에 따른 성공

# 1회 sampling 시 성공(앞면)-3/5(0.6)
# 3회 sampling 시 성공(앞면)-2/5(0.4)
# 5회 sampling 시 성공(앞면)-5/5(1.0) 
# [해설] 독립시행 n=10, 확률 p=0.5 일때 10번 시뮬레이션으로 추출추출 


# 2)주사위 확률실험 : 특정 눈금이 나오는 확률(ex:주사위 6번 던져서 2가 나올확률)  
sample3 = stats.binom.rvs(n=6, p=1/6, size=10) # 표본 10개 추출 
print('sample3 =', sample3)
# array([2, 1, 1, 1, 2, 1, 0, 1, 0, 2])
# 1회 sampling 시 성공(앞면)-2/6(0.333) : 6번 던져서 2번 성공 
# 9회 sampling 시 성공(앞면)-0/6(0.0) : 6번 던져서 모두 실패 


# 5) 표본의 확률변수 X 통계량
# 1) 확률변수 x 만들기 : 베르누이 시행 N의 베르누이분포변수 X 생성
n, p = 5, 1/6 # 5번 시행 
X = stats.binom.rvs(n, p, size=1000) # 1000번 시뮬레이션 : 표본 추출  
X  # 성공횟수 

# 2) 모집단 기댓값 : E(X) = np
Ex = n*p 
print('기댓값 =',Ex) # 기댓값 = 0.8333333333333333

# 3) 모집단 분산 : VAR(X) = np(1-p)
VARx = n*p *(1-p)
print('분산 = ', VARx) # 분산 =  0.6944444444444444

# 4) 확률변수 X의 기술통계 
print('표본의 확률변수 X의 검정통계량')
print(stats.describe(X))
'''
DescribeResult(nobs=1000, minmax=(0, 4), mean=0.881, variance=0.7015405405405405, skewness=0.7684897676063843, kurtosis=0.21244385790777187)
'''
#[해설] 시물레이션 수가 클 수록 모집단의 모수와 가까워진다. 



# 2. 이항검정 : 이항분포에 대한 가설검정 
# - 베르누이 확률 분포의 모수(P)에 대한 가설 검정  

# 연구환경 : 게임에 이길 확률(p)=40%, 시행회수(size)=100 일 때 성공횟수 ?  

# H0 : 게임에 이길 확률 P = 0.4 차이가 없다(다르지 않다). - default

# 1) 베르누이분포에서 표본추출(100개) 
p=0.4 # 모수(p)
size = 100 # 표본수 
X_bern = stats.bernoulli.rvs(p=p, size=size, random_state=None)
print(X_bern)
# 표본의 확률변수 = array([1, 0, 0, 1,..]  -> 성공(1), 실패(0)
'''
[1 0 0 1 0 0 1 1 0 0 1 0 1 0 0 1 1 0 0 0 0 1 1 1 1 0 0 0 0 0 0 0 1 1 0 0 1
 1 0 0 0 1 1 0 0 0 1 1 1 0 0 0 0 0 0 1 1 0 1 0 1 0 0 0 0 1 1 1 0 0 0 1 0 1
 1 1 0 1 0 0 0 1 0 1 0 1 0 0 1 0 0 0 1 1 1 1 1 0 0 1]
'''
X_binom = stats.binom.rvs(n=1, p=p, size=size)
print(X_binom)
'''
[1 0 0 0 1 0 0 0 0 1 1 0 1 0 0 0 1 0 1 0 1 0 1 0 0 1 0 0 1 0 0 1 0 1 0 1 1
 1 0 0 0 0 0 0 1 1 0 1 1 1 0 1 1 1 0 0 0 0 1 1 0 0 1 1 0 0 0 0 0 0 0 0 1 0
 0 0 1 1 1 0 1 0 1 0 1 0 0 0 1 0 0 0 1 1 1 1 1 1 1 0]
'''


# 2) 성공 횟수 반환(1인 경우 반환) : zero 제외  
print('bernoulli 성공회수 =', np.count_nonzero(X_bern)) # 성공횟수 = 43 
print('binom 성공회수 =', np.count_nonzero(X_binom)) # 성공횟수 = 43

# 3) 이항검정 
#help(stats.binom_test)
'''
binom_test(x, n=None, p=0.5, alternative='two-sided')
 x : 성공횟수, n : 시행회수, p : 귀무가설 확률(0.5), alternative='two-sided' : 양측검정 
''' 
x = np.count_nonzero(X_bern) # 성공회수
print('## 유의확률 ##')
pvalue = stats.binom_test(x=x, n=n, p=p, alternative='two-sided') # 성공회수, 시행회수 

print('x = %d, pvalue= %.5f'%(x, pvalue)) 
# x = 37, pvalue= 0.61007 -> 표본과 모수(p)과 차이가 없다. 
'''
[해설] 성공횟수(x)가 모수(40%)에 가까울 수록 pvalue 높고, 멀어질 수록 pvalue는 낮아진다. 
'''

# (4) 유의확률 vs 유의수준 
print('## 이항검정 결과 ##')
if pvalue > 0.05 : # 유의확률 > 유의수준 
    print(f"p-value({pvalue}) : 기존 성공률 40%와 차이가 없다.")
else:
    print(f"p-value({pvalue}) : 기존 성공률 40%와 차이가 있다.")
'''
[해설] 시행횟수 100회, 성공횟수 41회 일때 유의확률 : 61%
       이때 유의확률은 게임에 이길 확률 40% 가정하에 나온 확률이다.
       따라서 모집단의 모수(P=0.4)와 차이가 없다고 볼 수 있다.(귀무가설 채택) 
'''

# [이항검정 적용] 
# 150명의 합격자 중에서 남자 합격자가 62명일 때 
# 남여 합격률에 차이가 있다고 할수 있는가?


# H0 : 남여 합격률에 차이가 없다.(P=0.5)
print('## 유의확률 ##')
x = 62
pvalue = stats.binom_test(x=x, n=150, p=0.5, alternative='two-sided') # 성공횟수, 시행횟수 
print('n = %d, pvalue= %.5f'%(x, pvalue)) 
# n = 62, pvalue= 0.04087

# (2) 이항검정 
print('## 이항 검정 ##')
if pvalue > 0.05 : # 유의확률 > 유의수준 
    print(f"p-value({pvalue}) >= 0.05 : 남여 합격률에 차이가 없다.")
else:
    print(f"p-value({pvalue}) < 0.05 : 남여 합격률에 차이가 있다.")
    
'''    
## 유의확률 ##
n = 62, pvalue= 0.04087
## 이항 검정 ##
p-value(0.040868493866493945) < 0.05 : 남여 합격률에 차이가 있다.

[해설] 시행횟수=150, 성공횟수=62 일때 유의확률 : 4.087%       
 따라서 유의확률이 유의수준 0.05보다 작기 때문에 남여 합격률에 차이가 있다.
''' 










