# -*- coding: utf-8 -*-
"""
표준정규분포와 정규성 검정 
"""

from scipy import stats # 확률분포와 검정
import numpy as np # 분폭곡선을 위한 벡터 생성 
import matplotlib.pyplot as plt # 확률분포의 시각화 
    
#================================================
#  확률분포와 확률밀도함수 사용 
#================================================
# - 정규분포 (Normal : Gaussian) 예 

# 1. 정규분포 객체 : 확률분포 객체    
# loc : 기댓값(평균), scale : 표준편차 
norm_obj = stats.norm(loc=0, scale=1) # 정규분포 객체 생성 
#print(norm_obj) # objec info

# 2. 확률변수 x 만들기 : 정규분포의 확률밀도함수
N = 1000 # sample 수(시행 횟수) 
X = norm_obj.rvs(N) # N번 시뮬레이션 : 시행횟수 N을 이용하여 정규분포 확률변수 X 생성 
print(X)

# 3. 확률변수 x 시각화 : 히스토그램(단위:밀도)  
plt.hist(X, bins="auto", density=True, label="N(0,1)") 
plt.legend()
plt.show()
'''
1. 좌우대칭분포(평균 0 중심)
2. 전체 확률의 합 = 1
3. 표준편차와 확률 관계  
  -1 ~ +1 : 96%
  -2 ~ +2 : 95%
  -3 ~ +3 : 99%
'''

    
# 4. 확률밀도함수(밀도함수) : 확률밀도분포 곡선 추가
'''
확률변수 X가 x0 < X < x1 구간에 들어갈 확률 P(x0 < X < x1)로 정의하고,
식은 적분으로 나타낸다.
'''
# 분포곡선 vector data 
line = np.linspace(min(X), max(X), 100) # 분포곡선을 위한 line 벡터 
print('벡터 길이 =', len(line)) # 100

pdf = norm_obj.pdf(line) # 확률밀도함수 
# 확률밀도함수 : 확률변수 X가 x가 될 때의 확률을 반환하는 함수 

# 히스토그램 + 확률밀도분포 곡선
plt.hist(X, bins='auto', density=True, label="N(0,1)")
plt.plot(line, pdf, color='red') # 확률밀도분포 곡선
plt.title("확률밀도함수 ")
plt.xlabel("표준편차")
plt.ylabel("밀도(density)") 
plt.legend(loc='upper right')
plt.grid(True) # 격자 
plt.show()


# 5. 누적분포함수(분포함수) 
cdf = norm_obj.cdf(line) # 누적분포함수 
# 누적분포함수 : 확률변수 X가 x이하가 될때 까지의 누적확률 반환하는 함수   
print(cdf[-1]) # 누적분포함수의 마지막 원소의 값은 1에 근사한 값 

plt.plot(line, cdf, color='blue')
plt.title("누적분포함수 ")
plt.xlabel("표준편차")
plt.ylabel("밀도(density)") 
plt.grid(True) # 격자 
plt.show()


# 6. 정규성 검정 

# 1) 정규분포에서 표본추출     
norm_obj = stats.norm(loc=175, scale=5) # 정규분포 객체 생성 
 
N = 2000 # sample 수(시행 횟수) 
X = norm_obj.rvs(N) # N번 시뮬레이션 : 시행횟수 N을 이용하여 확률변수 X 만들기 
print(X)

# 2) 정규성 검정 : H0 : 정규분포와 차이가 없다.
print(stats.shapiro(X))
'''
ShapiroResult(statistic=0.9994271993637085, pvalue=0.9941558837890625)
'''
statistic, pvalue = stats.shapiro(X)
print('검정통계량 = ', statistic)

# 유의확률과 유의수준 -> 가설검정 
if pvalue >= 0.05 :
    print('정규분포와 차이가 없다.') # 정규분포와 차이가 없다.
else : 
    print('정규분포와 차이가 있다.')
        