# -*- coding: utf-8 -*-
from scipy import stats # 확률분포 + 검정
import numpy as np # 수치 처리 
import matplotlib.pyplot as plt # 확률분포의 시각화 


# 정규분포 N(75, 5^2)에서 70~80점 사이의 확률 구하기 

# 1. 정규분포 : N(75, 5^2)
st_norm = stats.norm(75, 5) # N(75, 5^2)

# 70~80점 사이의 확률 
prob = st_norm.cdf(80) - st_norm.cdf(70)
print('정규분포 확률 =',prob) # 0.6826894921370859

# 2. z분포 이용 
st_norm = stats.norm(0,1) # N(0,1)
 
#z = x_bar - mu / sigma
z1 = (70 - 75) / 5 # -1.0
z2 = (80 - 75) / 5 # 1.0

print('z1 누적확률 =', st_norm.cdf(z1)) # 0.8413447460685429
print('z2 누적확률 =', st_norm.cdf(z2)) # 0.15865525393145707

# # 70~80점 사이의 확률  : P(-1 < Z < 1)
prob2 = st_norm.cdf(z2) - st_norm.cdf(z1)  
print('z분포 확률 =', prob2) # prob2 = 0.6826894921370859


# 3. 구간 확률 
# x가 -3 ~ 3 일때 확률밀도함수를 이용하여 분포곡선   
x = np.linspace(-3, 3, 100)
plt.plot(x, st_norm.pdf(x)) # 확률밀도분포 곡선
# 0.0044318484119380075 ~ 0.0044318484119380075

interval = x[np.logical_and(x >= z1, x <= z2)] # P(-1 < Z < 1)
interval.min() # -1
interval.max() # 1


plt.fill_between(interval, y1=0, y2=st_norm.pdf(interval), alpha=0.5)
plt.show() 


# 4. 기타 정규분포에서 누적확률 시각화 

# 1) 0~n까지 누적확률 
# x가 -3 ~ 3 일때 확률밀도함수를 이용하여 분포곡선   
x = np.linspace(-3, 3, 100)
plt.plot(x, st_norm.pdf(x)) # 확률밀도분포 곡선
# 0.0044318484119380075 ~ 0.0044318484119380075

interval = x[x <= 1] # P(-3 < Z < 1)

plt.fill_between(interval, y1=0, y2=st_norm.pdf(interval), alpha=0.5)
plt.show() 


# 2) n이상 누적확률 
# x가 -3 ~ 3 일때 확률밀도함수를 이용하여 분포곡선   
x = np.linspace(-3, 3, 100)
plt.plot(x, st_norm.pdf(x)) # 확률밀도분포 곡선
# 0.0044318484119380075 ~ 0.0044318484119380075

interval = x[x >= z2] # P(1 < Z < 3)

plt.fill_between(interval, y1=0, y2=st_norm.pdf(interval), alpha=0.5)
plt.show() 






