"""
Z검정 : 모집단이 정규분포이고, 모분산을 알고 있는 경우 
"""

from scipy import stats # 확률분포와 검정
import numpy as np # 분폭곡선을 위한 벡터 생성 
import matplotlib.pyplot as plt # 확률분포의 시각화 


# 1. 정규분포에서 표본 추출     
mu = 30 # 모평균 
sigma = 5 # 모 표준편차 
st_norm = stats.norm(loc = mu, scale = sigma) # N(30, 5^2) 정규분포 객체 생성 

n = 50 # sample 수(시행 횟수) 
X = st_norm.rvs(n) # N번 시뮬레이션으로 확률변수 만들기 
print(X)

# 확률변수 x 시각화 : 히스토그램(단위:밀도)  
plt.hist(X, bins="auto", density=True, label="N(30, 5)") 
plt.legend()
plt.show()

# 2. Z검정 통계량 : z = (표본평균 - 모평균) / (표본표준편차 / 표본수 제곱근)
xbar = X.mean() # 표본평균  
Z = (xbar - mu ) / (sigma / np.sqrt(len(X)))  
print('Z =',Z) # -0.9370750585825375

# 3. Z검정 : 유의수준 알파 : 0.05 vs 유의확률 
# 귀무가설(H0) : 모평균 = 30 
alpha = 0.05

# 95% 신뢰수준에 따른 z값의 채택역   
interval = stats.norm().interval(1-alpha) # (-1.959963984540054, 1.959963984540054)
print('z값의 채택역 : ', interval)

# Z검정통계량과 채택역 비교 
if interval[0] < Z < interval[1] :
    print('귀무가설(H0) 채택')
else : 
    print('대립가설(H1) 채택')
    
















