'''
t검정 : t 분포에 대한 가설검정
 - t분포는 표본수가 30 이하인 경우의 정규분포  
 - t검정 : 모집단이 정규분포이고, 모분산이 알려지지 않은 경우
  
t 검정 유형 
  1. 단일표본 T검정(한 집단 평균 검정)
'''

from scipy import stats # test
import numpy as np # sampling


# 1. 단일표본 T검정 : 남자 평균 키(모평균) : 175.5cm -> 29명 표본추출 

# 1) 표본 추출 
sample_data = np.random.uniform(172,179, size=29) 
print(sample_data)
print(len(sample_data),'명') 

# 2) 정규성 검정 : H0 : 정규분포와 차이가 없다.
statistic, pvalue = stats.shapiro(sample_data)
print('pvalue =', pvalue)


# 3) T검정 : 단일집단 평균차이 검정 
help(stats.ttest_1samp)
statistic, pvalue = stats.ttest_1samp(sample_data, 175.5) # (a, popmean)
print('t검정 통계량 = %.5f, pvalue = %.5f'%(statistic, pvalue))
# t검정 통계량 = 1.255, pvalue = 0.21972

# 유의확률과 유의수준 비교 
alpha = 0.05
if pvalue >=  alpha:
    print('귀무가설(H0) 채택')
else : 
    print('대립가설(H1) 채택')
    
# 4) 기술통계 
print('평균 키 =', sample_data.mean()) # np.mean


# 5) 비모수 검정 : 웰콕스(wilcoxon)
statistic, pvalue = stats.wilcoxon(sample_data - 175.5) # (x-y)
print('wilcoxon 검정통계량 = %.5f, pvalue = %.5f'%(statistic, pvalue))
# t검정 통계량 = 192.00000, pvalue = 0.58137
'''
웹콕스(wilcoxon)는 T- 검정의 비모수 검정방법으로 
x, y 차이의 분포가 대칭인지 여부를 검정한다.
'''

