''' 
t 검정 유형 
  2. 독립표본 T검정(두 집단 평균 검정)
'''

from scipy import stats # test
import numpy as np # sampling


# 2. 독립표본 T검정 : 두 집단 평균 검정 : 남여 평균 점수 차이 검정

# 1) 표본추출  
method1_score = np.random.uniform(50, 100, size=30) # 교육방법1의 실기 점수 
method2_score = np.random.uniform(45, 95, size=30) # 교육방법2의 실기 점수

print('교육방법1의 실기 점수')
print(method1_score)
print('교육방법2의 실기 점수')
print(method2_score)


# 2) 동질성 검정 : H0 : 집단 간의 분포는 동일하다.
statistic, pvalue = stats.levene(method1_score, method2_score)
print('동질성 검정통계량 = %.3f, pvalue = %.5f'%(statistic, pvalue))

# 3) T검정 : 두 집단 평균 검정
statistic, pvalue = stats.ttest_ind(method1_score, method2_score)
print('t검정 통계량 = %.3f, pvalue = %.5f'%(statistic, pvalue))
# t검정 통계량 = 2.044, pvalue = 0.04547

# 4) 귀무가설 채택 or 기각 결정 
alpha = 0.05
if pvalue >=  alpha:
    print('귀무가설(H0) 채택')
else : 
    print('대립가설(H1) 채택')
    
# 대립가설(H1) 채택
# [해설] 여성 집단의 점수와 남성 집단의 점수의 평균 차이가 없다.

# 5) 기술통계 
print('교육방법1의 평균 점수 =', method1_score.mean()) 
print('교육방법2의 평균 점수 =', method2_score.mean())
'''
교육방법1의 평균 점수 = 78.05087119776154
교육방법2의 평균 점수 = 73.39100302578092
'''



