# -*- coding: utf-8 -*-

from scipy import stats #levene, f_oneway
import matplotlib.pyplot as plt # 요약통계량 시각화  
import numpy as np 
from statsmodels.formula.api import ols # formula 작성 
from statsmodels.stats.anova import anova_lm # 이원분산분석
import pandas as pd # DataFrame 만들기 


## 2. 이원분산분석

# 1) 표본추출  
age = np.random.randint(20, 50, size=100) # 20~49
time = np.random.randint(0, 2, size=100)  # 0~1
buy = np.random.randint(1, 16, size=100) # 1~15

# DataFrame 만들기 
sample = pd.DataFrame({'age':age, 'time':time, 'buy':buy})
print(sample.info())

# 연령대별 subset
age20 = sample[sample['age'] < 30]
age30 = sample[np.logical_and(sample['age'] > 30, sample['age'] < 40)]
age40 = sample[sample['age'] > 40] 

# 시간대별 subset
time_am = sample[sample['time'] == 0]
time_pm = sample[sample['time'] == 1]


# 2) 동질성 검정 : 연령대별 
statistic, pvalue = stats.levene(age20.buy, age30.buy, age40.buy)
print('동질성 검정통계량 = %.3f, pvalue = %.5f'%(statistic, pvalue))

# 상자그래프 시각화 : 요약통계량 시각화 
plot_data = [age20.buy, age30.buy, age40.buy]
ax = plt.boxplot(plot_data)
plt.show() # 연령대별 구매비용의 평균에 차이가 있음 

# 2) 동질성 검정 : 시간대별 
statistic, pvalue = stats.levene(time_am.buy, time_pm.buy)
print('동질성 검정통계량 = %.3f, pvalue = %.5f'%(statistic, pvalue))

# 상자그래프 시각화 : 요약통계량 시각화 
plot_data = [time_am.buy, time_pm.buy]
ax = plt.boxplot(plot_data)
plt.show() # 시간대별 구매비용의 평균에 차이가 있음 


# 3) F검정 : 일원분산분석
lm = ols(formula = 'buy ~ age + time', data=sample).fit()
print(anova_lm(lm))
'''
            df       sum_sq     mean_sq         F    PR(>F)
age        1.0   109.649997  109.649997  6.762797  0.010763
time       1.0    91.330659   91.330659  5.632930  0.019597
Residual  97.0  1572.729344   16.213705       NaN       NaN
'''
# [해설] 유의미한 수준에서 연령대별, 시간대별 구매비용에 차이가 있다고 볼 수 있다.

# 4) 기술통계 : 연령대별 
print('20대 평균 구매비용 =', age20['buy'].mean()) 
print('30대 평균 구매비용 =', age30['buy'].mean())
print('40대 평균 구매비용 =', age40['buy'].mean()) 
'''
20대 평균 구매비용 = 7.0
30대 평균 구매비용 = 6.225806451612903
40대 평균 구매비용 = 9.628571428571428
'''

# 4) 기술통계 : 시간대별 
print('오전 평균 구매비용 =', time_am['buy'].mean()) 
print('오후 평균 구매비용 =', time_pm['buy'].mean())
'''
오전 평균 구매비용 = 6.431818181818182
오후 평균 구매비용 = 8.75
'''
















