'''
 카이제곱 검정(chisquare test) 
  - 확률변수의 적합성 검정 - 일원  
  - 두 집단변수 간의 독립성 검정 - 이원 
  - 검정통계량(기대비율) = sum( (관측값 - 기댓값)**2 / 기댓값 )
  - 검정통계량 채택역 : -1.96 ~ +1.96  
'''

from scipy import stats # 확률분포 검정 
import numpy as np # 기대비율 계산  

# 카이제곱 검정(chisquare test) 
# 검정통계량(기대비율) = sum( (관측값 - 기댓값)**2 / 기댓값 )
# 검정통계량 채택역 : -1.96 ~ +1.96  

# 1. 일원 chi-square(1개 변수 이용) 
# 1) 적합성 검정 
# 귀무가설 : 관측치와 기대치는 차이가 없다.(게임에 적합하다.)
# 대립가설 : 관측치와 기대치는 차이가 있다.(게임에 적합하지 않다.) 

# (1) 관측도수와 기대도수 
real_data = [4, 6, 17, 16, 8, 9] # 관측치 
exp_data = [10,10,10,10,10,10] # 기대치 
chis = stats.chisquare(real_data, exp_data)
print('statistic = %.3f, pvalue = %.3f'%(chis)) 
# statistic = 14.200, pvalue = 0.014

# pvalue = 0.014 < 0.05 : 기각 
# [해설] 게임에 적합하지 않다.

# (2) 검정통계량(기대비율) = sum( (관측값 - 기댓값)**2 / 기댓값 )
real_arr = np.array(real_data)
exp_arr = np.array(exp_data)

statistic = sum( ((real_arr - exp_arr)**2) / exp_arr) 
print('검정통계량 =', statistic)


# 2) 선호도분석

# (1) 관측도수와 기대도수
real_data2 = [41, 49, 51, 71, 61] # 관측치 
exp_data2 = [51,51,51,51,51] # 기대치 
chis2 = stats.chisquare(real_data2, exp_data2)
print('statistic = %.3f, pvalue = %.3f'%(chis2)) 


# 2) 검정통계량(기대비율) = sum( (관측값 - 기댓값)**2 / 기댓값 )
real_arr = np.array(real_data2)
exp_arr = np.array(exp_data2)

statistic = sum( ((real_arr - exp_arr)**2) / exp_arr) 
print('검정통계량 =', statistic)



# 2. 이원 chi-square(2개 변수 이용) : 교육수준과 흡연 간의 독립성 검정
#귀무가설 : 교육수준과 흡연율 간에 관련성이 없다.(채택)
#연구가설 : 교육수준과 흡연율 간에 관련성이 있다.(기각)


import pandas as pd

# file 불러오기 
smoke = pd.read_csv("C:/PythonStatistics/data/smoke.csv")
print(smoke.info())

# <단계 1> 변수 빈도수 확인 
print(smoke)# education, smoking 변수
education = smoke.education
print(education.value_counts())
'''
1    211
3     92
2     52
'''
smoking = smoke.smoking
print(smoking.value_counts())
'''
2    141
1    116
3     98
'''

# <단계 2> 교차분할표
crosstab = pd.crosstab(index=education, columns=smoking)
print(crosstab)
'''
smoking     1   2   3
education            
1          51  92  68
2          22  21   9
3          43  28  21
'''

#<단계 3> 카이제곱 검정
chis2, pvalue, dof, E = stats.chi2_contingency(crosstab)
print('chis2 = %.3f, pvalue = %.6f'%(chis2, pvalue))
# chis2 = 18.911, pvalue = 0.000818

# [해설] 교육수준과 흡연율 간에 관련성이 있다.








