# -*- coding: utf-8 -*-
"""
scipy 패키지 이용 
 1. 단순선형회귀분석 
 2. 다중선형회귀분석 
"""

from scipy import stats # 회귀모델 
import pandas as pd # csv file 읽기 

# 1. 단순선형회귀분석 

# 1) file 불러오기
score_iq = pd.read_csv('C:/PythonStatistics/data/score_iq.csv')
print(score_iq.head())


# 2) 변수 선택
X = score_iq['iq'] # 독립변수 
Y = score_iq['score'] # 종속변수 

# 3) model 생성 
model = stats.linregress(X, Y)
print(model)
'''
LinregressResult(
    slope=0.6514309527270075, : x 기울기 
    intercept=-2.8564471221974657, : y 절편 
    rvalue=0.8822203446134699, : 설명력
    pvalue=2.8476895206683644e-50, : F검정 : 유의성검정 
    stderr=0.028577934409305443) : 표준오차 
'''

0.88 **2
# 4) 예측값과 오차 
# (1) 회귀계수 : 기울기와 절편 
a = model.slope # x 기울기
b = model.intercept # y 절편 
print('기울기 : {0:.5f}, 절편 : {1:.5f}'.format(a, b))

# (2) 회귀방정식 : Y 예측값 
fitted_values = (X * a) + b # 예측값

# (3) 잔차(오차)
err = Y - fitted_values
print('error')
print(err) 


# (4) 관측값 vs 예측값 
print('관측값 평균 : ', Y.mean())
print('적합값 평균 : ', fitted_values.mean())


# 5) 회귀모델 시각화 
from pylab import plot, title, legend, show
'''
plot : 산점도 
title, legend : 제목, 범례 
show : 차트 보이기 
'''

# 산점도 
plot(score_iq['iq'], score_iq['score'], 'b.')
# 회귀선 
plot(score_iq['iq'], fitted_values, 'r.-')
title('line regression') # 제목 
legend(['x y scatter', 'line regression']) # 범례 
show()

# exam03


# 2. 다중선형회귀분석 : formula 형식 
'''
y ~ x1 + x2 + ...
'''
from statsmodels.formula.api import ols

# x1=iq, x2=academy, x3=tv

# file load 
score_iq = pd.read_csv('C:/PythonStatistics/data/score_iq.csv')
print(score_iq.info()) 

# 1) 종속변수 vs 독립변수 상관계수 
corr = score_iq.corr()
print(corr['score'])


# 2) 모듈라(formula) 작성 
obj = ols(formula='score ~ iq + academy + tv', data = score_iq)
obj.exog_names # ['Intercept', 'iq', 'academy', 'tv']
obj.exog # [[  1., 140.,   2.,   0.]]

# 3) 모델 생성 
model = obj.fit()


# 4) 회귀계수값 확인 
print('회귀 계수값\n%s'%(model.params))
'''
Intercept      2.542755 - y절편 
alcohol        0.307923 - x1 기울기 
chlorides     -2.399620 - x2 기울기 
citric_acid    0.563065 - x3 기울기 
'''

# 5) 회귀분석 결과 제공  
print(model.summary()) 
'''
                            OLS Regression Results                            
==============================================================================
Dep. Variable:                  score   R-squared:                       0.946
Model:                            OLS   Adj. R-squared:                  0.945
Method:                 Least Squares   F-statistic:                     860.1
Date:                Mon, 22 Mar 2021   Prob (F-statistic):           1.50e-92
Time:                        15:41:10   Log-Likelihood:                -274.84
No. Observations:                 150   AIC:                             557.7
Df Residuals:                     146   BIC:                             569.7
Df Model:                           3                                         
Covariance Type:            nonrobust                                         
==============================================================================
                 coef    std err          t      P>|t|      [0.025      0.975]
------------------------------------------------------------------------------
Intercept     24.7223      2.332     10.602      0.000      20.114      29.331
iq             0.3742      0.020     19.109      0.000       0.335       0.413
academy        3.2088      0.367      8.733      0.000       2.483       3.935
tv             0.1926      0.303      0.636      0.526      -0.406       0.791
==============================================================================
Omnibus:                       36.802   Durbin-Watson:                   1.905
Prob(Omnibus):                  0.000   Jarque-Bera (JB):               57.833
Skew:                           1.252   Prob(JB):                     2.77e-13
Kurtosis:                       4.728   Cond. No.                     2.32e+03
==============================================================================
Notes:
[1] Standard Errors assume that the covariance matrix of the errors is correctly specified.
[2] The condition number is large, 2.32e+03. This might indicate that there are
strong multicollinearity or other numerical problems.
'''
'''
Residuals:
    Min      1Q  Median      3Q     Max 
-1.7373 -0.9351 -0.2833  0.9585  6.4727 

Coefficients:
            Estimate Std. Error t value Pr(>|t|)    
(Intercept) 24.72225    2.33185  10.602  < 2e-16 ***
x1           0.37420    0.01958  19.109  < 2e-16 ***
x2           3.20880    0.36742   8.733 5.26e-15 ***
x3           0.19257    0.30291   0.636    0.526    
---
Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1

Residual standard error: 1.532 on 146 degrees of freedom
Multiple R-squared:  0.9464,	Adjusted R-squared:  0.9453 
F-statistic: 860.1 on 3 and 146 DF,  p-value: < 2.2e-16
'''

# 6) 다중공선성 확인 
from statsmodels.stats.outliers_influence import variance_inflation_factor

x_data = score_iq[['iq', 'academy', 'tv']]# x변수 데이터프레임 객체  
x_data.shape # (150, 3) 

vif = pd.DataFrame() # 빈 데이터프레임 
# VIF 칼럼 추가 
vif["VIF Factor"] = [variance_inflation_factor(obj.exog, i+1) 
                     for i in range(x_data.shape[1])]
# Features 칼럼 추가 
vif["features"] = x_data.columns
print(vif) 
'''
   VIF Factor features
0    1.917392       iq
1   12.579732  academy
2   10.494479       tv
'''

# academy와 tv가 10 이상이므로 서로 다중공선성에 문제가 있는 것으로 보고 
# academy와 tv 중 하나를 drop 하고 회귀모델을 만들면 다중공선성 문제가 해결됨 

# 7) 다중공선성 문제 해결 : academy 제거 
obj = ols(formula='score ~ iq + tv', data = score_iq)
model = obj.fit()

x_data = score_iq[['iq', 'tv']]# x변수 데이터프레임 객체  
 

vif = pd.DataFrame() # 빈 데이터프레임 
# VIF 칼럼 추가 
vif["VIF Factor"] = [variance_inflation_factor(obj.exog, i+1) for i in range(2)]
# Features 칼럼 추가 
vif["features"] = x_data.columns
print(vif)
'''
   VIF Factor features
0    1.520366       iq
1    1.520366       tv
'''

# 8) 다중공선성 해결 후 회귀모델 결과 
print(model.summary())

'''
                            OLS Regression Results                            
==============================================================================
Dep. Variable:                  score   R-squared:                       0.918
Model:                            OLS   Adj. R-squared:                  0.917
Method:                 Least Squares   F-statistic:                     828.0
Date:                Tue, 23 Mar 2021   Prob (F-statistic):           9.58e-81
Time:                        15:13:28   Log-Likelihood:                -306.36
No. Observations:                 150   AIC:                             618.7
Df Residuals:                     147   BIC:                             627.8
Df Model:                           2                                         
Covariance Type:            nonrobust                                         
==============================================================================
                 coef    std err          t      P>|t|      [0.025      0.975]
------------------------------------------------------------------------------
Intercept     26.7989      2.852      9.395      0.000      21.162      32.436
iq             0.4520      0.021     21.081      0.000       0.410       0.494
tv            -2.2537      0.142    -15.897      0.000      -2.534      -1.974
==============================================================================
Omnibus:                       43.309   Durbin-Watson:                   2.485
Prob(Omnibus):                  0.000   Jarque-Bera (JB):               70.387
Skew:                           1.493   Prob(JB):                     5.20e-16
Kurtosis:                       4.531   Cond. No.                     2.30e+03
==============================================================================

Notes:
[1] Standard Errors assume that the covariance matrix of the errors is correctly specified.
[2] The condition number is large, 2.3e+03. This might indicate that there are
strong multicollinearity or other numerical problems.
'''


# 9) 다중공선성 문제 해결 : tv 제거 
obj = ols(formula='score ~ iq + academy', data = score_iq)
model = obj.fit()

x_data = score_iq[['iq', 'academy']]# x변수 데이터프레임 객체  

vif = pd.DataFrame() # 빈 데이터프레임 
# VIF 칼럼 추가 
vif["VIF Factor"] = [variance_inflation_factor(obj.exog, i+1) for i in range(2)]
# Features 칼럼 추가 
vif["features"] = x_data.columns
print(vif)
'''
   VIF Factor features
0    1.822462       iq
1    1.822462  academy
'''

# 10) 다중공선성 해결 후 회귀모델 결과 
print(model.summary())



# 회귀분석의 가정에 적합한 독립변수 선택 
#The condition number is large, Cond. No. : 2.32e+03., 0.945, DW:1.905 -> 3개 
# AIC:                             557.7
#The condition number is large, Cond. No. : 2.3e+03., 0.917, DW:2.485 -> 2개(iq, tv) 
# AIC:                             618.7
#condition number is large, Cond. No. : 2.18e+03. - 0.945, DW : 1.913 -> 2개(iq, academy) 
# AIC:                             556.1







